@@ -1,7 +1,8 @@
-from ..losses import GenericPairLoss
 import torch
-from ..utils import loss_and_miner_utils as lmu
+
 from ..distances import CosineSimilarity
+from ..losses import GenericPairLoss
+from ..utils import loss_and_miner_utils as lmu
 
 
 class IntraPairVarianceLoss(GenericPairLoss):
@@ -48,7 +49,7 @@ class IntraPairVarianceLoss(GenericPairLoss):
             return (1 - eps) * mean_sim - pairs
         return pairs - (1 + eps) * mean_sim
 
-    def sub_loss_names(self):
+    def _sub_loss_names(self):
         return ["pos_loss", "neg_loss"]
 
     def get_default_distance(self):
