@@ -1,6 +1,8 @@
 import torch
+
+from ..utils import common_functions as c_f
+from ..utils import loss_and_miner_utils as lmu
 from .generic_pair_loss import GenericPairLoss
-from ..utils import loss_and_miner_utils as lmu, common_functions as c_f
 
 
 class LiftedStructureLoss(GenericPairLoss):
@@ -18,10 +20,11 @@ class LiftedStructureLoss(GenericPairLoss):
 
         if len(a1) > 0 and len(a2) > 0:
             pos_pairs = pos_pairs.unsqueeze(1)
-            n_per_p = (
+            n_per_p = c_f.to_dtype(
                 (a2.unsqueeze(0) == a1.unsqueeze(1))
-                | (a2.unsqueeze(0) == p.unsqueeze(1))
-            ).type(dtype)
+                | (a2.unsqueeze(0) == p.unsqueeze(1)),
+                dtype=dtype,
+            )
             neg_pairs = neg_pairs * n_per_p
             keep_mask = ~(n_per_p == 0)
 
