@@ -1,5 +1,6 @@
-from .base_miner import BaseMiner, BaseTupleMiner, BaseSubsetBatchMiner
 from .angular_miner import AngularMiner
+from .base_miner import BaseMiner, BaseSubsetBatchMiner, BaseTupleMiner
+from .batch_easy_hard_miner import BatchEasyHardMiner
 from .batch_hard_miner import BatchHardMiner
 from .distance_weighted_miner import DistanceWeightedMiner
 from .embeddings_already_packaged_as_triplets import EmbeddingsAlreadyPackagedAsTriplets
@@ -8,3 +9,4 @@ from .maximum_loss_miner import MaximumLossMiner
 from .multi_similarity_miner import MultiSimilarityMiner
 from .pair_margin_miner import PairMarginMiner
 from .triplet_margin_miner import TripletMarginMiner
+from .uniform_histogram_miner import UniformHistogramMiner
