@@ -1,7 +1,9 @@
-from .base_trainer import BaseTrainer
-from ..utils import common_functions as c_f, loss_and_miner_utils as lmu
 import torch
 
+from ..utils import common_functions as c_f
+from ..utils import loss_and_miner_utils as lmu
+from .base_trainer import BaseTrainer
+
 
 class TwoStreamMetricLoss(BaseTrainer):
     def calculate_loss(self, curr_batch):
@@ -50,5 +52,5 @@ class TwoStreamMetricLoss(BaseTrainer):
         else:
             return lmu.get_all_triplets_indices(labels, labels.clone())
 
-    def allowed_mining_funcs_keys(self):
-        return ["tuple_miner"]
+    def modify_schema(self):
+        self.schema["mining_funcs"].keys = ["tuple_miner"]
